
import java.awt.image._

import Array._
import math._

/**
 * <code>FourierFilter</code> is a subclass of abstract class <code>Filter</code>. It operates
 * on images pixel by pixel in frequency domain.
 */
class FourierFilter extends Filter {
  
  /**
   * The only public function of the class takes <code>BufferedImge</code> specified by
   * <code>image</code> and operates a filter on it specified by <code>filterName</code>.
   * Returns the result image after the filter is operated on the image in frequency domain.
   * 
   * @param image       the image that will operated on stored as <code>BufferedImage</code>
   * @param filterName  the name of the filter that will be operated on <code>image</code>
   */
  def applyFilter(image: BufferedImage, filterName: String) : BufferedImage = {
    // your code here
    image
  }
  
  /**
   * The function performs a two dimensional FFT on the given arrays <code>gRe</code> and
   * <code>gIm</code> storing the results in arrays <code>resRe</code> and <code>resIm</code>
   * respectively. If the FFT is inverse it is specified with <code>inverse</code>.
   * 
   * @param width    width of the image and the arrays operated with
   * @param height   height of the image and the arrays operated with
   * @param inverse  whether the FFT is performed in inverse, <code>false</code> when moving from
   *                 spatial space to frequency space, <code>true</code> otherwise
   * @param gRe	     the real part of the starting array
   * @param gIm	     the imaginary part of the starting array
   * @param resRe    the real part of the result array
   * @param resIm    the imaginary part of the result array
   */
  private def FFT2D(width:Int, height:Int, inverse:Boolean,
                    gRe:Array[Array[Array[Float]]], gIm:Array[Array[Array[Float]]],
                    resRe:Array[Array[Array[Float]]], resIm:Array[Array[Array[Float]]]) : Unit = {
    // your code here
  }
  
  /**
   * One dimensional FFT. The direction is defined with <code>columnDirection</code> and the x and
   * y values are defined accordingly.
   * 
   * @param columnDirection  identifies the dimension to be transformed, when <code>true</code> FFT
   *                         is performed on columns, when <code>false</code> FFT is performed on rows
   * @param len              contains direction's size e.g. width or height
   * @param resRe            the result array for real values
   * @param resIm            the result array for imaginary values
   * @param inverse          whether the FFT is inverse, <code>false</code> when moving from spatial
   *                         space to frequency space, <code>true</code> otherwise
   */
  private def FFT(columnDirection: Boolean, len: Int, resRe: Array[Array[Array[Float]]],
                  resIm: Array[Array[Array[Float]]], inverse: Boolean) : Unit = {
    
    val log2 = (log(len) / log(2)).toInt // log_2(len)
    for(n <- 0 until len) { // rows or columns
      for(c <- 0 until 3) { // the color components
        var cosine = -1.0f // cos(Pi)
        var sine = 0.0f // sin(Pi)
        var l1, l2 = 1
        for(l <- 0 until log2) {
          l1 = l2
          l2 *= 2

          var odd = 1.0f
          var even = 0.0f
          for(j <- 0 until l1) {
            val range: Range = Predef.intWrapper(j).until(len)
            val byRange: Range = range.by(l2)
            for(i <- byRange) { // i goes from j until len with increment by l2
              val i1 = i + l1

              var x1, x = n
              var y1 = i1
              var y = i
              if (!columnDirection) {
                x1 = i1; x = i
                y1 = n; y = n
              }

              var t1 = odd*resRe(x1)(y1)(c) - even*resIm(x1)(y1)(c)
              var t2 = odd*resIm(x1)(y1)(c) + even*resRe(x1)(y1)(c)

              resRe(x1)(y1)(c) = resRe(x)(y)(c) - t1
              resIm(x1)(y1)(c) = resIm(x)(y)(c) - t2
              resRe(x)(y)(c) = resRe(x)(y)(c) + t1
              resIm(x)(y)(c) = resIm(x)(y)(c) + t2
            }
            val z =  odd * cosine - even * sine
            even = odd * sine + even * cosine
            odd = z
          }
          sine = sqrt((1.0 - cosine) / 2.0).toFloat
          if(!inverse) { sine = -sine }
          cosine = sqrt((1.0 + cosine) / 2.0).toFloat
        }
      }
    }
    
    //if (!inverse) {
    //  for (x <- 0 until len) {
    //    for (y <- 0 until len) {
    //      for (c <- 0 until 3) {
    //        resRe(x)(y)(c) /= len
    //        resIm(x)(y)(c) /= len
    //      }
    //    }
    //  }
    //}
  }
  
  /**
   * The function operates filter specified by <code>name</code> to a given picture given in
   * arrays <code>re</code> and <code>im</code> containing the real and imaginary parts of the
   * image respectively.
   * 
   * @param width   width of the image and the arrays operated with
   * @param height  height of the image and the arrays operated with
   * @param name    name of the filter to be operated with
   * @param re      the real part of the image the filter is operated on
   * @param im      the imaginary part of the image the filter is operated on
   */
  private def operateFilter(width:Int, height:Int, name:String,
                            re:Array[Array[Array[Float]]], im:Array[Array[Array[Float]]]) : Unit = {
    // your code here
  }
  
 /**
   * Moves the values in <code>arr</code> so that corners of the image become centered.
   * 
   * @param arr     Array that will be operated on
   * @param width   the width of the <code>arr</code>
   * @param height  the height of the <code>arr</code>
   */
  private def moveCornersToCenter(arr:Array[Array[Array[Float]]],
                                  width:Int, height:Int) : Array[Array[Array[Float]]] = {
    // your code here
    arr
  }
  
  /**
   * Checks if the integer given as parameter is power of two.
   * 
   * @param num  the number to be checked to be the power of two
   * @return	 <code>true</code> when <code>num</code> is power of two otherwise <code>false</code>
   */
  private def isPowerOfTwo(num:Int) : Boolean = {
    // your code here
    false
  }

   /**
   * Fills array given in parameter <code>res</code> with values in <code>org</code> in
   * bit-reversed order.
   * 
   * The resulting array is filled with values from the original array so that the values are
   * moved to a bit-reversed index, e.g
   * 0 -> 0000 0000 -> 0000 0000 -> 0
   * 1 -> 0000 0001 -> 1000 0000 -> 128
   * 2 -> 0000 0010 -> 0100 0000 -> 64
   * 3 -> 0000 0011 -> 1100 0000 -> 192
   * and so forth.
   * 
   * @param width   width of the image and the arrays operated with
   * @param height  height of the image and the arrays operated with
   * @param res     the result array to be filled with values in <code>org</code>
   * @param org	    the original array whose values are moved to <code>res</code>
   */
  private def bitReverseArray(width:Int, height:Int, res:Array[Array[Array[Float]]],
                              org:Array[Array[Array[Float]]]) : Unit = {
    // your code here
  }
  
  /**
   * Reverses <code>num</code> as follows:
   * 0 -> 0000 0000 -> 0000 0000 -> 0
   * 1 -> 0000 0001 -> 1000 0000 -> 128
   * 2 -> 0000 0010 -> 0100 0000 -> 64
   * 3 -> 0000 0011 -> 1100 0000 -> 192
   * and so forth.
   * 
   * @param num  a number to be reversed
   * @return     the bit-reversed number
   */
  private def bitReverse(num:Int) : Int = {
    // your code here
    0
  }
  
}
