
import java.awt.Color
import java.awt.image._
import java.io.File

import javax.imageio.ImageIO
import javax.swing.BorderFactory
import javax.swing.ImageIcon

import collection.mutable.ListBuffer
import io.Source
import swing._
import event._

object View extends SimpleSwingApplication {
  
  def top = new MainFrame() {
    title = "Photogram"
    
    //TODO You should change these!
    /* THESE FILE LOCATIONS SHOULD BE CHANGED ACCORDING TO YOUR PERSONAL SETTINGS. */
    val imageLocation = "src/kuva.png"
    val pixelFilterLocation = "src/pixelfilter.txt"
    val fourierFilterLocation = "src/fourierfilter.txt"
    
    
    /* NO NEED TO DO ANYTHING WITH THINGS BELOW THIS POINT. */  
    var img:BufferedImage = ImageIO.read(new File(imageLocation)) // load image from file
    var icn = new ImageIcon(img) // create image icon with loaded image to be showed
    val imageLabel = new Label("", icn, Alignment.Center)
    
    // filter objects
    val pixel_filter = new PixelFilter()
    val fourier_filter = new FourierFilter()
    
    // list buffer for reading buttons from files
    val buf = new ListBuffer[Tuple2[Button, String]]
    
    // reading pixel filter buttons
    for (line <- Source.fromFile(pixelFilterLocation).getLines()) {
      buf += new Tuple2(new Button(line), line)
    }
    val buttons = buf.toList
    
    // reading fourier filter buttons
    buf.clear()
    for (line <- Source.fromFile(fourierFilterLocation).getLines()) {
      buf += new Tuple2(new Button(line), line)
    }
    val buttons_f = buf.toList
    
    // looping through button lists and adding reactions
    buttons.foreach(x => listenTo(x._1))
    buttons_f.foreach(x => listenTo(x._1))
    
    reactions += {
      case ButtonClicked(b) =>
        for (x <- buttons if x._1 == b) { img = pixel_filter.applyFilter(img, x._2) }
        if (!buttons_f.isEmpty) {
          for (x <- buttons_f if x._1 == b) { img = fourier_filter.applyFilter(img, x._2) } }
        icn.setImage(img)
        this.repaint()
    }
    
    // same for back to original button
    val original = new Button("back to original")
    listenTo(original)
    reactions += {
      case ButtonClicked(`original`) =>
        img = ImageIO.read(new File(imageLocation))
        icn.setImage(pixel_filter.applyFilter(img, "original"))
        this.repaint()
    }
    
    // setting minimum size and centered position for mainframe
    minimumSize = new Dimension(img.getWidth(), img.getHeight() + buttons.length*20)
    centerOnScreen()
    
    contents = new BoxPanel(Orientation.Vertical) { // contents for the main frame
      contents += new GridPanel(1,3) { // adding the image
        contents += imageLabel
      }
      
      contents += new GridPanel(1,1) { // adding back to original button
        contents += original
      }
      
      contents += new GridPanel(1, 2) { // adding headers to buttons
        contents += new Label("Pixel by Pixel")
        contents += new Label("Fourier Transform")
      }
      
      contents += new GridPanel(1, 2) { // adding buttons for filter buttons
        contents += new BoxPanel(Orientation.Vertical) {
          contents ++= (for (b <- buttons) yield { b._1 }) // pixel
          border = BorderFactory.createLineBorder(Color.black)
        }
        contents += new BoxPanel(Orientation.Vertical) {
          contents ++= (for (b <- buttons_f) yield { b._1 }) // fourier
          border = BorderFactory.createLineBorder(Color.black)
        }
      }
    }
    
  }
  
}
